﻿using Com.Bocom.OpenApi.Utils;
using Kivii;
using Kivii.Finances.Entities;
using Kivii.Finances;
using Kivii.Finances.Transforms;
using Kivii.Linq;
using Kivii.Text;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using static Com.Bocom.OpenApi.AddContractOrderRequestV2.AddContractOrderRequestV2Biz;
using static System.Net.Mime.MediaTypeNames;
using System.Reflection;
using System.Web.Caching;

namespace Com.Bocom.OpenApi
{
    public static class Extension
    {
        public static AddContractOrderResponseV2 RequestAddContractOrderV2(Settlement settlement, out string err)
        {
            if (settlement == null)
            {
                err = string.Empty;
                return null;
            }
            DefaultBocomClient client = new DefaultBocomClient(Configs.APP_ID, Configs.MY_PRIVATE_KEY, Configs.APIGW_PUBLIC_KEY, BocomConstants.ENCRYPT_TYPE_RSA_AND_AES);
            /**
			* 测试环境可以忽略SSL证书告警，生产环境不可忽略
			*/
            client.IgnoreSslHostnameVerifier();

            var bizKvid = settlement.Kvid.ToString();
            var bizId = settlement.BizId;
            var request = new AddContractOrderRequestV2();
            request.SetServiceUrl(Configs.APIGW_URL_ADDRESS + Configs.AddContractOrderRequestV2);

            var bizContent = new AddContractOrderRequestV2.AddContractOrderRequestV2Biz();
            bizContent.org_id = Configs.ORG_ID;
            bizContent.bat_id = bizId;
            bizContent.cont_order_list = new List<ContOrder>();
            var contorder = new ContOrder();
            contorder.cont_id = bizKvid;
            contorder.org_id = bizContent.org_id;
            contorder.ori_payer = settlement.PayerName;
            //contorder.ori_ac = "";
            //contorder.ori_addr = "仅记录";
            //contorder.ori_tel_no = "仅记录";
            //contorder.ori_other_info1 = "仅记录";
            //contorder.ori_other_info2 = "仅记录";
            //contorder.ori_other_info3 = "仅记录";
            contorder.ori_itm_nme = settlement.BizId;
            contorder.ori_amt = settlement.AmountPlan.ToString();
            contorder.ori_offset_amt = (settlement.AmountPlan - settlement.Amount).ToString();
            contorder.ori_act_amt = settlement.Amount.ToString();
            //contorder.attach_code = "";
            contorder.remark = settlement.Remark;
            //contorder.offset_remark = "测试2";
            contorder.payment_period = $"{DateTime.Now.AddMonths(-2).ToString("yyyy-MM-dd")}~{DateTime.Now.AddMonths(7).ToString("yyyy-MM-dd")}";
            contorder.agreement_type = "-1";
            bizContent.cont_order_list.Add(contorder);

            request.SetBizContent(bizContent);
            var response = client.Execute(request, Guid.NewGuid().ToString().Replace("-", ""));
            var rtns = new AddContractOrderResponseV2();
            if (response.IsSuccess())
            {
                rtns = response as AddContractOrderResponseV2;
                err = string.Empty;
                return rtns;
            }
            else
            {
                err = response.GetRspMsg();
                return null;
            }

        }

        public static DelContractOrderResponseV1 RequestDelContractOrderResponseV1(Settlement settlement, out string err)
        {
            if (settlement == null || settlement.Metadata.IsNullOrEmpty() || !settlement.Metadata.ContainsKey("BocomBillId"))
            {
                err = string.Empty;
                return null;
            }
            var billId = settlement.Metadata["BocomBillId"];
            //Console.WriteLine($"---------------BocomBillId:{billId}-------------------");
            DefaultBocomClient client = new DefaultBocomClient(Configs.APP_ID, Configs.MY_PRIVATE_KEY, Configs.APIGW_PUBLIC_KEY, BocomConstants.ENCRYPT_TYPE_RSA_AND_AES);
            /**
			* 测试环境可以忽略SSL证书告警，生产环境不可忽略
			*/
            client.IgnoreSslHostnameVerifier();
            var request = new DelContractOrderRequestV1();
            request.SetServiceUrl(Configs.APIGW_URL_ADDRESS + Configs.DelContractOrderRequestV1);

            var bizContent = new DelContractOrderRequestV1.DelContractOrderRequestV1Biz();
            bizContent.org_id = Configs.ORG_ID;
            bizContent.bill_id = billId;
            request.SetBizContent(bizContent);
            var response = client.Execute(request, Guid.NewGuid().ToString().Replace("-", ""));
            var rtns = new DelContractOrderResponseV1();
            if (response.IsSuccess())
            {
                rtns = response as DelContractOrderResponseV1;
                err = string.Empty;
                //Console.WriteLine($"---------------DelContractOrder:Success-------------------");
                return rtns;
            }
            else
            {
                err = response.GetRspMsg();
                //Console.WriteLine($"---------------err:{err}-------------------");
                return null;
            }
        }

        /// <summary>
        /// 智慧记账自动核销结算
        /// </summary>
        /// <param name="payments"></param>
        /// <param name="settlement"></param>
        /// <param name="payingMethod"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        /// <exception cref="Exception"></exception>
        public static bool AutoPay(List<Payment> payments, Settlement settlement, List<PayingMethod> payingMethods, IDbConnection conn = null)
        {
            payments.ThrowIfNullOrEmpty("请传入付款信息!");
            settlement.ThrowIfNull("请传入结算信息!");
            if (payments.Exists(o => o.Type != PaymentType.Split)) throw new Exception("付款类型不支持!");
            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<Payment>();
            }
            var accountBiz = settlement.Currency.GetBizAccount();


            var dynamicSplitPayments = new List<Payment>();
            var allPays = new List<Pay>();
            var allAccountDetails = new List<AccountDetail>();

            foreach (var payingMethod in payingMethods)
            {
                var amount = payingMethod.Amount;
                var payment = payments.FirstOrDefault(o => o.Kvid == payingMethod.Kvid);
                (payment == null).ThrowIfTrue("未找到付款数据!");
                if (payment.Amount - payment.AmountUsed < amount) amount = payment.Amount - payment.AmountUsed;
                (payingMethod.Amount > 0 && payingMethod.Amount > payment.Amount - payment.AmountUsed).ThrowIfTrue($"所选付款方式余额不足,可用余额:{payment.Amount - payment.AmountUsed}");
                if (payingMethod.Amount > 0 && payingMethod.Amount < amount) amount = payingMethod.Amount;//外部指定分摊多少钱就按照指定的费用进行分摊 否则就按照能使用的最大金额进行分摊
                var settlements = new List<Settlement>();
                settlements.Add(settlement);
                var dynamicPayPayment = settlements.Paying(payment, accountBiz, out var pays, out var accountDetail, amount, payingMethod.Remark);
                allPays.AddRange(pays);
                dynamicSplitPayments.Add(dynamicPayPayment);
                allAccountDetails.Add(accountDetail);
            }

            IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            if (useTransaction) trans = conn.OpenTransaction();
            try
            {
                foreach (var item in allPays)
                {
                    conn.Insert(item);
                }
                foreach (var item in dynamicSplitPayments)
                {
                    item.SerialNumber = item.GetSubSerialNumber(conn);
                    conn.Insert(item);//创建DynamicSplitPayment
                }
                allAccountDetails.ForEach(o => o.Insert(conn));//创建账户明细
                foreach (var item in payments)
                {
                    item.RecalculateAmountUsed(conn);
                    if (item.Type == PaymentType.Split) item.RecalculateAssignAmount(conn);
                }
                if (!settlement.OnlyProperties.IsNullOrEmpty()) conn.UpdateOnly(settlement);//更新了Paykvid和AmountPayment

                trans?.Commit();

                return true;
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
        }
    }
}
